package Apache::FileLister;

use strict;
use warnings;

use Apache::Constants qw/OK/;
use Dir::List;
use YAML::LoadFileCached;
use Hash::Merge qw/merge/;
use File::Basename;
use Template;
use RPM2;
use Date::Format;

our $VERSION = (qw$Revision: 2.2 $ )[-1];

sub search($) {
	my $self = shift;
	my @search_result;
	if(open(FH, "find " . $self->{config}->{root} . $self->{current} . " -type f 2> /dev/null | ")) {
		my @files;
		push @files, $_ foreach(<FH>);
		chomp(@files);
		my $i = 0;
		foreach my $filename (sort @files) {
			my $skip = 0;
			foreach(@{$self->{config}->{exclude}}) {
				if($filename =~ /$_/) { $skip = 1 };	
			}
			next if $skip;
			if($filename =~ /$self->{search_string}/) {
				$i++;
				my %file;

				$file{id} = $i;

				$file{filename} = $filename;
				$file{filename} =~ s/$self->{config}->{root}//;
				$file{directory} = dirname($file{filename});

				push @search_result, \%file;
				if($i >= $self->{config}->{max_search_results}) {
					close(FH);
					$self->{max_search_results_hit} = 1;
					return \@search_result || 'undef';
				}
			}
		}
		close(FH);
	} else {
		warn "Could not open filehandle for find-pipe";
		return undef;
	}
	return \@search_result || 'undef';
}

sub rpminfo($) {
	my $self = shift;
	my $info;
	my $file = $self->{config}->{root} . $self->{rpmfile};
	if(-f $file) {
		my $pkg = RPM2->open_package($file);
		#warn Dumper(keys %RPM2::header_tag_map);
		#foreach(keys %RPM2::header_tag_map) {
		#	s/^RPMTAG_//;
		#	eval {
		#		warn Dumper("$_:" . $pkg->tag($_)) if $pkg->tag($_);
		#	};
		#}
		foreach(qw/name version vendor release epoch buildhost group source sourcerpm size license packager url summary description buildtime disturl distribution /) {
			$info->{$_} = $pkg->tag($_) if $pkg->tag($_);
			$info->{$_} =~ s/</&lt;/g if $info->{$_};
			$info->{$_} =~ s/>/&gt;/g if $info->{$_};
			$info->{$_} =~ s/\n/<br>\n/g if $_ eq 'description';
		}
		my $i;
		foreach($pkg->changelog()) {
			$i++;
			$_->{name} =~ s/</&lt;/g if $_->{name};
			$_->{name} =~ s/>/&gt;/g if $_->{name};
			$_->{text} =~ s/</&lt;/g if $_->{text};
			$_->{text} =~ s/>/&gt;/g if $_->{text};
			$_->{text} =~ s/\n/<br>\n/g if $_->{text};
			push @{$info->{changelog}}, $_;
			return $info if $i >= $self->{config}->{max_changelog_entries};
		}
		return $info;
	} else {
		return undef;
	}
}

sub handler {
	my $self = { };
	$self->{r} = Apache::Request->new(shift);
	$self->{tt} = Template->new(
		ABSOLUTE => 1
	);

	$Template::Stash::SCALAR_OPS->{time2str} = sub { time2str('%Y-%m-%d %H:%M:%S', $_[0]); };
	$Template::Stash::SCALAR_OPS->{changelog_time2str} = sub { time2str('%a %b %d %Y', $_[0]); };

	$self->{docroot} = $self->{r}->document_root();
	$self->{docroot} .= '/' unless $self->{docroot} =~ /\/$/;
	$self->{system_path} = $self->{docroot}.'../mod_perl/_system/';

	my $default_config = $self->{system_path}.'config/FileLister.default.conf';
	my $local_config = $self->{system_path}.'config/FileLister.local.conf';
	if(-f $local_config) {
		Hash::Merge::set_behavior('RIGHT_PRECEDENT');
		$self->{config} = merge(LoadFileCached($default_config), LoadFileCached($local_config));
	} else {
		$self->{config} = LoadFileCached($default_config);
	}

	$self->{current} = $self->{r}->param('current') || '/';
	# Kill double dots.
	$self->{current} =~ s/\.\.//g;
	# Kill double slashes.
	$self->{current} =~ s/$/\//g;
	$self->{current} =~ s/(\/\/)/\//g;

	if($self->{config}->{enable_search}) {
		if($self->{search_string} = $self->{r}->param('search_string')) {
			$self->{search_string} = "" unless $self->{search_string};
			$self->{search_string} =~ s/\.\.//g;
			$self->{search_string} =~ s/(\/\/)/\//g;
			$self->{search_string} =~ s/\*/\.\*/;
			if($self->{search_string}) {
				$self->{search_result} = search($self);
			}
		}
	}
	if($self->{r}->param('rpminfo')) {
		$self->{rpmfile} = $self->{r}->param('rpminfo');
		$self->{rpmfile} =~ s/\.\.//g;
		$self->{rpmfile} =~ s/(\/\/)/\//g;
		$self->{rpmfile} =~ s/\*/\.\*/;
		if($self->{rpmfile}) {
			$self->{rpminfo} = rpminfo($self);
		}
	}

	@{$self->{dirs}} = split "/", $self->{current};

	$self->{dir} = Dir::List->new({
		exclude			=> $self->{config}->{exclude} || [ qw// ],
		use_cache		=> $self->{config}->{caching}->{enabled} || 0,
		cache_root		=> $self->{config}->{caching}->{path} || '/tmp/FileLister',
		cache_expires	=> $self->{config}->{caching}->{expire} || '5 minutes',
	});

	$self->{r}->send_http_header('text/html');

	$self->{headline_path} = "";
	$self->{path_line} = '<a href="?current=/">H O M E</a>';

	foreach (@{$self->{dirs}}) {
		$self->{headline_path} .= "/".$_ unless $_ eq '';
		$self->{path_line} .= '<a href="?current=' . $self->{headline_path} . '/">' . $_ . '</a>' . ' / ';
	}

	$self->{dirinfo} = $self->{dir}->dirinfo($self->{config}->{root}.$self->{current});

	foreach (keys %{$self->{dirinfo}->{files}}) {
		if($self->{dirinfo}->{files}->{$_}->{internal_type}) {
			if($self->{dirinfo}->{files}->{$_}->{internal_type} eq 'rpm') {
				if(-f $self->{config}->{root} . $self->{current} . $_ . '.txt') {
					$self->{dirinfo}->{files}->{$_}->{note} = 1;
				}
			}
		}

		if(-f $self->{config}->{root} . $self->{current} . $_ . '.stickynote') {
			$self->{dirinfo}->{files}->{$_}->{stickynote} = 1;
		}
	}

	$self->{title} = 'Apache-FileLister ' .$VERSION . ' @ ' . $self->{r}->hostname();
	$self->{version} = $VERSION;
	if($self->{config}->{debug}) {
		use Data::Dumper;
		$self->{dump} = Dumper($self);
	}

	$self->{tt}->process($self->{system_path}.'templates/main.tpl', $self) || print $self->{tt}->error();

	return OK;
}

1;

__END__

=head1 NAME

FileLister - Perl module for Apache for listing files/directories.

=head1 SYNOPSIS

In your httpd.conf, make a (for example) a vhost like this
(I expect, you have copied FileLister to /var/www/filelister.company.com/):

	<VirtualHost *:80>
		ServerName		filelister.company.com
		DocumentRoot	/var/www/filelister.company.com/html

		Alias	/download	/var/ftp
		Alias	/css		/var/www/filelister.company.com/html/css
		Alias	/images		/var/www/filelister.company.com/html/images

		<IfModule mod_perl.c>
			PerlRequire		/var/www/filelister.company.com/mod_perl/startup.pl

			<Location /mod_perl>
				SetHandler perl-script
				PerlSendHeader On
				PerlHandler FileLister
			</Location>
		</IfModule>
	</VirtualHost>

=head1 DESCRIPTION

This module provides you with a nice interface for browsing directories and
downloading files. It's much better than the normal Apache dirlisting...

EXPORT

Nothing. mod_perl directly calles the handler() function.

=head1 BUGS

None, that I'm aware of.

=head1 TODO

See README.

=head1 SEE ALSO

Apache::Constants
Dir::List
YAML::LoadFileCached
Hash::Merge
File::Basename
Template
RPM2
Date::Format

=head1 AUTHOR

Oliver Falk, E<lt>oliver@linux-kernel.atE<gt>

=head1 COPYRIGHT

Copyright (C) 2005 by Oliver Falk

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.6 or,
at your option, any later version of Perl 5 you may have available.

=cut
